package partisan.weforge.xyz.pulse

import android.content.ContentResolver
import android.os.Bundle
import android.provider.ContactsContract
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import partisan.weforge.xyz.pulse.databinding.FragmentContactsBinding

class ContactsFragment : Fragment() {

    private var _binding: FragmentContactsBinding? = null
    private val binding get() = _binding!!

    private lateinit var prefs: Preferences

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentContactsBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onResume() {
        super.onResume()
        (requireActivity() as? MainActivity)?.apply {
            setAppBarTitle(getString(R.string.settings_name), getString(R.string.whitelist_name))
            setupPopupToggle(true, prefs.isBlacklistEnabled) { isChecked ->
                prefs.isBlacklistEnabled = isChecked
                binding.contactRecycler.isEnabled = isChecked
                binding.contactRecycler.alpha = if (isChecked) 1f else 0.4f
            }
        }

        // Initial state
        binding.contactRecycler.isEnabled = prefs.isBlacklistEnabled
        binding.contactRecycler.alpha = if (prefs.isBlacklistEnabled) 1f else 0.4f
    }

    override fun onPause() {
        super.onPause()
        (requireActivity() as? MainActivity)?.setupPopupToggle(false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        prefs = Preferences(requireContext())

        val contacts = getContacts()
        val adapter = ContactAdapter(prefs, contacts)
        binding.contactRecycler.layoutManager = LinearLayoutManager(requireContext())
        binding.contactRecycler.adapter = adapter
    }

    private fun getContacts(): List<ContactEntry> {
        val resolver: ContentResolver = requireContext().contentResolver
        val projection = arrayOf(
            ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME,
            ContactsContract.CommonDataKinds.Phone.NUMBER
        )

        val cursor = resolver.query(
            ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
            projection,
            null,
            null,
            "${ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME} ASC"
        )

        val results = mutableListOf<ContactEntry>()

        cursor?.use {
            val nameIndex = it.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME)
            val numberIndex = it.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER)

            while (it.moveToNext()) {
                val name = it.getString(nameIndex) ?: continue
                val number = it.getString(numberIndex) ?: continue
                results.add(ContactEntry(name, number))
            }
        }

        return results
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}
